local objEscape = { ['"'] = '\\"', ["\b"] = "\\b", ["\f"] = "\\f", ["\n"] = "\\n", ["\r"] = "\\r", ["\t"] = "\\t", ["\\"] = "\\\\" };
local objVariable = { ["boolean"] = true, ["nil"] = true, ["number"] = true, ["string"] = true, ["table"] = true };

local function is_array(objInput)
	local intReturn = 0;

	if ( type(objInput) == "table" ) then
		for mixKey, _ in pairs(objInput) do
			if ( type(mixKey) == "number" ) then intReturn = intReturn + 1; else return false; end
			if ( not objInput[intReturn] ) then return false; end
		end
	else return false; end

	return true, intReturn;
end

local function is_variable(mixVariable)
	return objVariable[type(mixVariable)] ~= nil;
end

local function json_escape(strInput)
	return strInput:gsub('["%\\%z\001-\031]', function(strCharacter)
		if ( json_escape_list[strCharacter] ) then
			return json_escape_list[strCharacter];
		else
			return string.format("\\u%04x", strCharacter:byte());
		end
	end);
end

function json_encode(mixInput)
	local strType = type(mixInput);

	if ( strType == "string" ) then
		return '"'.. json_escape(mixInput) ..'"';
	elseif ( strType == "boolean" or strType == "number" ) then
		return tostring(mixInput);
	elseif ( strType == "table" ) then
		local objReturn, intArray, intCount = {}, is_array(mixInput);

		if ( intArray ) then
			for intKey = 1, intCount do
				if ( is_variable(mixInput[intKey]) ) then
					table.insert(objReturn, json_encode(mixInput[intKey]));
				end
			end

			return "[".. table.concat(objReturn, ",") .."]";
		else
			for strKey, mixValue in pairs(mixInput) do
				if ( is_variable(strKey) and is_variable(mixValue) ) then
					table.insert(objReturn, '"'.. strKey ..'":'.. json_encode(mixValue));
				end
			end

			return "{".. table.concat(objReturn, ",") .."}";
		end
	end

	return "null";
end
